@@ -13,13 +13,11 @@
 #include <KConfigGroup>
 #include <KLocalizedString>
 #include <KSharedConfig>
-#include <KStatusNotifierItem>
 
 #include <QDBusConnection>
-#include <QDBusInterface>
-#include <QDBusReply>
-#include <QLocale>
-#include <QThread>
+#include <QDBusMessage>
+#include <QDBusPendingCallWatcher>
+#include <QQmlEngine>
 #include <QXmlStreamReader>
 
 AlarmModel *AlarmModel::instance()
@@ -28,6 +26,15 @@ AlarmModel *AlarmModel::instance()
     return singleton;
 }
 
+AlarmModel *AlarmModel::create(QQmlEngine *qmlEngine, QJSEngine *jsEngine)
+{
+    Q_UNUSED(qmlEngine);
+    Q_UNUSED(jsEngine);
+    auto *model = instance();
+    QQmlEngine::setObjectOwnership(model, QQmlEngine::CppOwnership);
+    return model;
+}
+
 AlarmModel::AlarmModel(QObject *parent)
     : QAbstractListModel{parent}
     , m_interface{new org::kde::kclock::AlarmModel(QStringLiteral("org.kde.kclockd"), QStringLiteral("/Alarms"), QDBusConnection::sessionBus(), this)}
@@ -56,25 +63,47 @@ AlarmModel::AlarmModel(QObject *parent)
 void AlarmModel::load()
 {
     // load from dbus
-    QDBusInterface *interface = new QDBusInterface(QStringLiteral("org.kde.kclockd"),
-                                                   QStringLiteral("/Alarms"),
-                                                   QStringLiteral("org.freedesktop.DBus.Introspectable"),
-                                                   QDBusConnection::sessionBus(),
-                                                   this);
-    QDBusReply<QString> reply = interface->call(QStringLiteral("Introspect"));
-    if (reply.isValid()) {
-        QXmlStreamReader xml(reply.value());
-        while (!xml.atEnd()) {
-            xml.readNext();
-            if (xml.name() == QStringLiteral("node") && xml.attributes().hasAttribute(QStringLiteral("name"))) {
-                if (xml.attributes().value(QStringLiteral("name")).toString().indexOf(QStringLiteral("org")) == -1) {
-                    this->addAlarmInternal(xml.attributes().value(QStringLiteral("name")).toString());
+
+    m_busy = true;
+    Q_EMIT busyChanged(m_busy);
+
+    // Unfortunately, Qt DBus doesn't support DBus ObjectManager.
+    QDBusMessage message = QDBusMessage::createMethodCall(QStringLiteral("org.kde.kclockd"),
+                                                          QStringLiteral("/Alarms"),
+                                                          QStringLiteral("org.freedesktop.DBus.Introspectable"),
+                                                          QStringLiteral("Introspect"));
+    QDBusPendingCall call = QDBusConnection::sessionBus().asyncCall(message);
+    auto *watcher = new QDBusPendingCallWatcher(call, this);
+    connect(watcher, &QDBusPendingCallWatcher::finished, this, [this, watcher] {
+        QDBusPendingReply<QString> reply = *watcher;
+
+        if (reply.isError()) {
+            // TODO show error in UI.
+            qWarning() << "Failed to fetch alarms" << reply.error().name() << reply.error().message();
+        } else {
+            beginResetModel();
+            QXmlStreamReader xml(reply.value());
+            while (!xml.atEnd()) {
+                xml.readNext();
+                if (xml.isStartElement() && xml.name() == QLatin1String("node")) {
+                    const auto name = xml.attributes().value(QLatin1String("name"));
+                    if (!name.isEmpty() && !name.contains(QLatin1String("org"))) {
+                        addAlarmInternal(name.toString());
+                    }
                 }
             }
+            endResetModel();
         }
-    }
 
-    interface->deleteLater();
+        m_busy = false;
+        Q_EMIT busyChanged(m_busy);
+        watcher->deleteLater();
+    });
+}
+
+bool AlarmModel::busy() const
+{
+    return m_busy;
 }
 
 QHash<int, QByteArray> AlarmModel::roleNames() const
@@ -84,7 +113,7 @@ QHash<int, QByteArray> AlarmModel::roleNames() const
 
 QVariant AlarmModel::data(const QModelIndex &index, int role) const
 {
-    if (!index.isValid() || index.row() >= alarmsList.count()) {
+    if (!index.isValid() || index.row() >= alarmsList.count() || role != AlarmRole) {
         return QVariant();
     }
 
@@ -94,7 +123,9 @@ QVariant AlarmModel::data(const QModelIndex &index, int role) const
 
 int AlarmModel::rowCount(const QModelIndex &parent) const
 {
-    Q_UNUSED(parent);
+    if (parent.isValid()) {
+        return 0;
+    }
     return alarmsList.size();
 }
 
