@@ -13,13 +13,23 @@
 #include <QAbstractListModel>
 #include <QObject>
 
+#include <qqmlregistration.h>
+
+class QJSEngine;
+class QQmlEngine;
+
 class AlarmModel : public QAbstractListModel
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_SINGLETON
+
     Q_PROPERTY(bool connectedToDaemon READ connectedToDaemon NOTIFY connectedToDaemonChanged)
+    Q_PROPERTY(bool busy READ busy NOTIFY busyChanged)
 
 public:
     static AlarmModel *instance();
+    static AlarmModel *create(QQmlEngine *qmlEngine, QJSEngine *jsEngine);
 
     enum {
         AlarmRole,
@@ -27,6 +37,9 @@ public:
 
     void load();
 
+    bool busy() const;
+    Q_SIGNAL void busyChanged(bool busy);
+
     int rowCount(const QModelIndex &parent) const override;
     QVariant data(const QModelIndex &index, int role) const override;
     Qt::ItemFlags flags(const QModelIndex &index) const override;
@@ -58,6 +71,7 @@ private:
     org::kde::kclock::AlarmModel *m_interface;
     QDBusServiceWatcher *m_watcher;
     bool m_connectedToDaemon = false;
+    bool m_busy = false;
 
     explicit AlarmModel(QObject *parent = nullptr);
     QList<Alarm *> alarmsList;
