@@ -14,9 +14,13 @@
 #include <QTimer>
 #include <QUuid>
 
+#include <qqmlintegration.h>
+
 class Timer : public QObject
 {
     Q_OBJECT
+    QML_ELEMENT
+    QML_UNCREATABLE("Managed by TimerModel")
     Q_PROPERTY(int length READ length WRITE setLength NOTIFY lengthChanged)
     Q_PROPERTY(QString lengthPretty READ lengthPretty NOTIFY lengthChanged)
     Q_PROPERTY(int elapsed READ elapsed NOTIFY elapsedChanged)
@@ -25,7 +29,7 @@ class Timer : public QObject
     Q_PROPERTY(QString label READ label WRITE setLabel NOTIFY labelChanged)
     Q_PROPERTY(QString commandTimeout READ commandTimeout WRITE setCommandTimeout NOTIFY commandTimeoutChanged)
     Q_PROPERTY(bool running READ running NOTIFY runningChanged)
-    Q_PROPERTY(bool looping READ looping NOTIFY loopingChanged)
+    Q_PROPERTY(bool looping READ looping WRITE setLooping NOTIFY loopingChanged)
     Q_PROPERTY(bool ringing READ ringing NOTIFY ringingChanged)
 
 public:
@@ -52,6 +56,7 @@ public:
     bool running() const;
 
     bool looping() const;
+    void setLooping(bool looping);
 
     bool ringing() const;
 
