@@ -6,11 +6,16 @@
 #include "settingsmodel.h"
 
 #include <QLocale>
+#include <QQmlEngine>
 #include <QStandardPaths>
 #include <QTimeZone>
 #include <QUrl>
 
+#include <KConfigGroup>
 #include <KLocalizedString>
+#include <KSharedConfig>
+
+#include <KSvg/Svg>
 
 UtilModel *UtilModel::instance()
 {
@@ -18,6 +23,15 @@ UtilModel *UtilModel::instance()
     return singleton;
 }
 
+UtilModel *UtilModel::create(QQmlEngine *qmlEngine, QJSEngine *jsEngine)
+{
+    Q_UNUSED(qmlEngine);
+    Q_UNUSED(jsEngine);
+    auto *model = instance();
+    QQmlEngine::setObjectOwnership(model, QQmlEngine::CppOwnership);
+    return model;
+}
+
 UtilModel::UtilModel(QObject *parent)
     : QObject{parent}
 {
@@ -69,29 +83,21 @@ QString UtilModel::timeToRingFormatted(const long long &timestamp)
     int day = remaining / (24 * 3600);
     int hour = remaining / 3600 - day * 24;
     int minute = remaining / 60 - day * 24 * 60 - hour * 60;
-    QString arg;
+    QStringList parts;
     if (day > 0) {
-        arg += i18np("%1 day", "%1 days", day);
+        parts << i18np("%1 day", "%1 days", day);
     }
     if (hour > 0) {
-        if (day > 0 && minute > 0) {
-            arg += i18n(", ");
-        } else if (day > 0) {
-            arg += i18n(" and ");
-        }
-        arg += i18np("%1 hour", "%1 hours", hour);
+        parts << i18np("%1 hour", "%1 hours", hour);
     }
     if (minute > 0) {
-        if (day > 0 || hour > 0) {
-            arg += i18n(" and ");
-        }
-        arg += i18np("%1 minute", "%1 minutes", minute);
+        parts << i18np("%1 minute", "%1 minutes", minute);
     }
 
     if (day <= 0 && hour <= 0 && minute <= 0) {
         return i18n("Alarm will ring in under a minute");
     } else {
-        return i18n("Alarm will ring in %1", arg);
+        return i18n("Alarm will ring in %1", QLocale().createSeparatedList(parts));
     }
 }
 
@@ -140,4 +146,51 @@ QString UtilModel::displayTwoDigits(const qint64 &amount)
     return QStringLiteral("%1").arg(amount, 2, 10, QLatin1Char('0'));
 }
 
+QString UtilModel::repeatFormat(int dayOfWeek) const
+{
+    if (!dayOfWeek) {
+        return i18nc("Repeat", "Only once");
+    }
+    if (dayOfWeek == (1 << 7) - 1) {
+        return i18nc("Repeat", "Everyday");
+    }
+    if (dayOfWeek == (1 << 5) - 1) {
+        return i18nc("Repeat", "Weekdays");
+    }
+
+    QLocale locale;
+
+    QList<QString> days;
+    days.reserve(7);
+    int lastDay = -1;
+    for (int day = 0; day < 7; ++day) {
+        if (dayOfWeek & (1 << day)) {
+            // 0 in QLocale is Sunday.
+            const QString dayName = locale.standaloneDayName(day + 1, QLocale::ShortFormat);
+            days.append(dayName);
+            lastDay = day;
+        }
+    }
+
+    // Single day, show full name.
+    if (days.count() == 1) {
+        return locale.standaloneDayName(lastDay + 1, QLocale::LongFormat);
+    } else {
+        return days.join(i18n(", "));
+    }
+}
+
+void UtilModel::applyPlasmaImageSet(KSvg::Svg *svg)
+{
+    if (!m_plasmaImageSet) {
+        m_plasmaImageSet = new KSvg::ImageSet(this);
+        m_plasmaImageSet->setBasePath(QStringLiteral("plasma/desktoptheme"));
+    }
+
+    KSharedConfig::Ptr plasmaRc = KSharedConfig::openConfig(QStringLiteral("plasmarc"));
+    const auto themeGroup = plasmaRc->group(QStringLiteral("Theme"));
+    m_plasmaImageSet->setImageSetName(themeGroup.readEntry("name", QString()));
+    svg->setImageSet(m_plasmaImageSet);
+}
+
 #include "moc_utilmodel.cpp"
