@@ -8,16 +8,18 @@
 
 #include "timer.h"
 #include "alarmplayer.h"
+#include "timermodel.h"
 
 #include <QFile>
 #include <QStandardPaths>
 
-Timer::Timer(int length, const QString &label, const QString &commandTimeout, bool running, QObject *parent)
+Timer::Timer(int length, const QString &label, bool looping, const QString &commandTimeout, bool running, QObject *parent)
     : QObject{parent}
     , m_uuid{QUuid::createUuid()}
     , m_length{length}
     , m_label{label}
     , m_commandTimeout{commandTimeout}
+    , m_looping{looping}
 {
     init();
 
@@ -58,13 +60,13 @@ void Timer::init()
     connect(&Utilities::instance(), &Utilities::needsReschedule, this, &Timer::reschedule);
 
     // initialize notification
-    m_notification->setIconName(QStringLiteral("kclock"));
+    m_notification->setIconName(QStringLiteral("org.kde.kclock"));
     m_notification->setTitle(i18n("Timer complete"));
     m_notification->setText(i18n("Your timer %1 has finished!", label()));
     m_notification->setUrgency(KNotification::HighUrgency);
     m_notification->setAutoDelete(false); // don't auto-delete when closing
 
-    auto defaultAction = m_notification->addDefaultAction(i18n("View"));
+    auto defaultAction = m_notification->addDefaultAction(i18nc("@action:button", "Dismiss"));
     connect(defaultAction, &KNotificationAction::activated, this, &Timer::dismiss);
 
     connect(m_notification, &KNotification::closed, this, &Timer::dismiss);
@@ -73,9 +75,6 @@ void Timer::init()
     QDBusConnection::sessionBus().registerObject(QStringLiteral("/Timers/") + m_uuid.toString(QUuid::Id128),
                                                  this,
                                                  QDBusConnection::ExportScriptableContents | QDBusConnection::ExportAllProperties);
-    connect(this, &QObject::destroyed, [this] {
-        QDBusConnection::sessionBus().unregisterObject(QStringLiteral("/Timers/") + m_uuid.toString(QUuid::Id128), QDBusConnection::UnregisterNode);
-    });
 }
 
 QJsonObject Timer::serialize()
@@ -96,9 +95,7 @@ void Timer::toggleRunning()
 
 void Timer::toggleLooping()
 {
-    m_looping = !m_looping;
-    Q_EMIT loopingChanged();
-
+    setLooping(!m_looping);
     TimerModel::instance()->save();
 }
 
@@ -137,9 +134,15 @@ void Timer::setLength(int length)
         Q_EMIT lengthChanged();
 
         TimerModel::instance()->save();
+        reschedule();
     }
 }
 
+void Timer::addMinute()
+{
+    setLength(m_length + 60);
+}
+
 QString Timer::label() const
 {
     return m_label;
@@ -175,6 +178,14 @@ bool Timer::looping() const
     return m_looping;
 }
 
+void Timer::setLooping(bool looping)
+{
+    if (m_looping != looping) {
+        m_looping = looping;
+        Q_EMIT loopingChanged();
+    }
+}
+
 bool Timer::running() const
 {
     return m_running;
@@ -263,7 +274,7 @@ void Timer::ring()
 
     Utilities::pauseMprisSources();
 
-    AlarmPlayer::instance().setSource(findRingSoundLocation());
+    AlarmPlayer::instance().setSource(TimerModel::instance()->defaultAudioLocation());
     AlarmPlayer::instance().play();
 
     m_ringing = true;
@@ -290,18 +301,4 @@ void Timer::reschedule()
     }
 }
 
-QUrl Timer::findRingSoundLocation()
-{
-    const auto locations = QStandardPaths::standardLocations(QStandardPaths::GenericDataLocation);
-    const QString path = QStringLiteral("/sounds/freedesktop/stereo/alarm-clock-elapsed.oga");
-    QString alarmSoundPath;
-
-    for (const auto &directory : locations) {
-        if (QFile(directory + path).exists()) {
-            return QUrl(directory + path);
-        }
-    }
-
-    return QUrl();
-}
 #include "moc_timer.cpp"
